# -*- coding: UTF-8 -*-
import sys
from datetime import datetime, date
from artix.mapper.object_mapper import ObjectMapper
from upload.converter.smagPlugins.xml.domain.artix_entities import *
from upload.converter.smagPlugins.xml.dto.ukm_entities import *

"""
Маппер из сущности ukm в сущность artix
"""


class UkmToArtixTmcMapper(object):
    tnvd_mapper = dict()
    measure_mapper = dict()
    tnvds_from_xml = dict()

    tmc_mapping = {
        'name': lambda entity: entity.name.replace("&amp;", "&"),
        'articul': lambda entity: entity.article,
        'inventcode': lambda entity: entity.article,
        'inventgroup': lambda entity: entity.groupId,
        'barcodes': lambda entity: UkmToArtixTmcMapper.map_barcode(entity.barcode),
        'tmctype': lambda entity: UkmToArtixTmcMapper.get_tmctype(entity.TNVDcode, entity.egaisType) if int(
            entity.egaisType) >= 1 else 0,
        'measurecode': lambda entity: UkmToArtixTmcMapper.get_measure_code(entity.measure),
        'taxgroupcode': lambda entity: entity.taxgroupId,
        'paymentobject': lambda entity: UkmToArtixTmcMapper.get_paymentobject(entity.egaisType, entity.SubExcise),
        'opmode': lambda entity: UkmToArtixTmcMapper.get_opmode(entity.egaisType),
        'price': None,  # цена не указывается в справочнике
        'barcode': None  # ШК грузим списком
    }

    barcode_mapping = {
        'barcode': lambda entity: entity.id,
        'cquant': lambda entity: 1 if entity.quantity and int(entity.quantity) == 0 else entity.quantity,
        'code': lambda entity: entity.article,
        'name': lambda entity: entity.name
    }

    price_mapping = {
        'price': lambda entity: entity.value,
        'minprice': lambda entity: entity.minprice,
        'minretailprice': lambda entity: entity.mrc if entity.mrc else entity.minprice,
        'barcode': lambda entity: entity.article,
        'documentid': lambda entity: 1,
        'doctype': lambda entity: 1,
        'effectivedate': lambda entity: datetime.combine(date.today(), datetime.min.time()).strftime(
            "%Y-%m-%d %H:%M:%S")
    }

    tprice_mapping = {
        'pricecode': lambda entity: entity.code,
        'name': lambda entity: entity.name,
        'price': lambda entity: entity.value,
        'barcode': lambda entity: entity.article,
        'documentid': lambda entity: 1,
        'effectivedate': lambda entity: datetime.combine(date.today(), datetime.min.time()).strftime(
            "%Y-%m-%d %H:%M:%S")
    }

    alcocode_mapping = {
        "alcocode": lambda entity: entity.AlcCode,
        'tmccode': lambda entity: entity.article,
    }

    tmc_group_mapping = {
        'groupcode': lambda entity: entity.id,
        'parentgroupcode': lambda entity: entity.owner,
        'groupname': lambda entity: str(entity.name).replace("&amp;", "&")
    }

    tax_mapping = {
        'code': lambda entity: entity.id,
        'name': lambda entity: entity.name,
        'rate': lambda entity: str(entity.percent).replace('%', '')
    }

    tmc_kit_mapping = {
        'barcode': lambda entity: entity.article,
        'name': lambda entity: entity.article
    }

    tmc_kit_item_mapping = {
        'tmccode': lambda entity: entity.article,
        'quantity': lambda entity: entity.quantity,
        'tmckitbarcode': lambda entity: entity.parentcode
    }

    subexcise_to_paymentobject_by_egaisType = {
        "0": lambda subExcise: 2 if int(subExcise) == 1 else 0,
        "1": lambda subExcise: 2,
        "3": lambda subExcise: 31 if int(subExcise) == 1 else 33,
        "4": lambda subExcise: 31 if int(subExcise) == 1 else 33
    }

    opmode_by_egaisType = {
        "0": "0",
        "1": "128",
        "3": "0",
        "4": "0"
    }

    mapper = ObjectMapper()
    mapper.create_map(UkmTmc, Tmc, tmc_mapping)
    mapper.create_map(UkmBarcode, Barcode, barcode_mapping)
    mapper.create_map(UkmGroup, TmcGroup, tmc_group_mapping)
    mapper.create_map(ProductAlcCode, AlcoCode, alcocode_mapping)
    mapper.create_map(UkmTax, TmcTax, tax_mapping)
    mapper.create_map(UkmPrice, Price, price_mapping)
    mapper.create_map(UkmTprice, Tprice, tprice_mapping)
    mapper.create_map(UkmKit, TmcKit, tmc_kit_mapping)
    mapper.create_map(UkmKitItem, TmcKitItem, tmc_kit_item_mapping)

    @classmethod
    def get_tmctype(cls, tnvd_group_id, egais_type):
        """
        получение типа товара по коду тнвд из справочника и по полю egaisType
        сомнительно, но допустим
        """

        if not tnvd_group_id and int(egais_type) == 1:
            # для алкогольных товаров в справочнике не заполняется код тнвд
            return 1
        # при отключенной настройке маппинг не будет считан, тогда просто парсим egaisType
        if len(cls.tnvd_mapper) == 0:
            return 7 if egais_type == 3 or egais_type == 4 else egais_type
        # по id записи, указанной в товаре получаем код группы тнвд
        # из списка, считанного из xml-файла
        code = cls.tnvds_from_xml.get(tnvd_group_id, None)
        # по коду группы тнвд получаем tmctype (из маппинга, указанного в настройках)
        # маппинг вида  <tmctype>=code1,code2,code3
        if code:
            for k, v in cls.tnvd_mapper.items():
                codes = v.split(',')
                if code in codes:
                    return k
        # по умолчанию возвращаем просто признак маркированного товара
        return 7

    @classmethod
    def get_measure_code(cls, measure_name):
        """
        получение кода единицы измерения из маппинга, указанного в настройках
        <код артикс>=<название из ukm>
        """
        if measure_name is not None:
            # во избежание проблем с кириллицей на python2
            if sys.version_info.major == 2:
                measure_name = measure_name.encode("UTF-8")
            for k, v in cls.measure_mapper.items():
                if v == measure_name:
                    return k
        # по умолчанию возвращаем 1
        return 1

    @classmethod
    def get_paymentobject(cls, egaisType, subExcise):
        """
        egaisType == 0 && subExcise == 0 =>  0
        egaisType == 0 && subExcise == 1 =>  2
        egaisType == 1 && subExcise == 0 =>  2
        egaisType == 1 && subExcise == 1 =>  2
        egaisType == 3 && subExcise == 0 =>  33
        egaisType == 3 && subExcise == 1 =>  31
        egaisType == 4 && subExcise == 0 =>  33
        egaisType == 4 && subExcise == 1 =>  31
        """
        return cls.subexcise_to_paymentobject_by_egaisType[egaisType](subExcise if subExcise else 0)

    @classmethod
    def get_opmode(cls, egaisType):
        """
        egaisType == 0 =>  0
        egaisType == 1 => 128
        egaisType == 3 => 0
        egaisType == 4 => 0
        """
        return cls.opmode_by_egaisType[egaisType]

    @classmethod
    def map_tmc(cls, ukm_tmcs, tnvds, measure_map=None, tnvd_map=None):
        """
        маппинг товара с поиском ед. измерения и типа товара (tmcType)
        """

        cls.measure_mapper = measure_map if measure_map is not None else {}
        cls.tnvds_from_xml = tnvds if tnvds is not None else {}
        cls.tnvd_mapper = tnvd_map if tnvd_map is not None else {}

        tmcs = list()
        for ukm_tmc in ukm_tmcs:
            ukm = UkmTmc(**ukm_tmc)
            for bcode in ukm.barcode:
                bcode['name'] = ukm.name
                bcode['article'] = ukm.article

            tmc = cls.mapper.map(ukm, Tmc)  # type: Tmc
            tmcs.append(tmc)
        return tmcs

    @classmethod
    def map_barcode(cls, ukm_barcode):
        """
        маппинг штрих-кодов
        """
        return [cls.mapper.map(UkmBarcode(**b), Barcode) for b in ukm_barcode]

    @classmethod
    def map_prices(cls, ukm_prices):
        """
        маппинг переоценок
        """
        return [cls.mapper.map(UkmPrice(**ukm_price), Price) for ukm_price in ukm_prices]

    @classmethod
    def map_tprices(cls, ukm_prices):
        """
        маппинг доп.цен
        """
        return [cls.mapper.map(UkmTprice(**ukm_price), Tprice) for ukm_price in ukm_prices]

    @classmethod
    def map_alcocode(cls, ukm_alcocodes):
        """
        маппинг алкокодов
        """
        return [cls.mapper.map(ProductAlcCode(**code), AlcoCode) for code in ukm_alcocodes]

    @classmethod
    def map_groups(cls, ukm_groups):
        """
        маппинг групп товаров
        """
        return [cls.mapper.map(UkmGroup(**group), TmcGroup) for group in ukm_groups]

    @classmethod
    def map_tax(cls, ukm_taxes):
        """
        маппинг налогов
        """
        return [cls.mapper.map(UkmTax(**tax), TmcTax) for tax in ukm_taxes]

    @classmethod
    def map_kit(cls, ukm_kits):
        """
        маппинг комплектов товаров и сущностей внутри них
        """
        kits = [UkmKit(**ukm_kit) for ukm_kit in ukm_kits]
        kit_items = list()
        for items in list(k.subItem for k in kits):
            kit_items.extend(items)
        ukm = [UkmKitItem(**i) for i in kit_items]
        mapped_kits = [cls.mapper.map(k, TmcKit) for k in kits]
        mapped_kit_items = [cls.mapper.map(item, TmcKitItem) for item in ukm]
        return mapped_kits, mapped_kit_items
