# -*- coding: UTF-8 -*-
import json
from datetime import date
from datetime import datetime

"""
artix-объекты для конвертера ukm-xml
для object_mapper нужен дефолтный конструктор, так что все поля в сигнатуре считаются None

TODO: когда откажемся от python2 переделать на @dataclasses!!!
"""


class BaseArtixDomainEntity(object):

    def as_dict(self):
        """
        возвращаем json со всеми вложенными объектами
        """
        return json.loads(json.dumps(self, default=lambda o: o.__dict__))


class Barcode(BaseArtixDomainEntity):
    """
    штрих-код
    """

    def __init__(self, code=None, barcode=None, name=None, cquant=None):
        self.code = code
        self.barcode = barcode
        self.name = name
        self.cquant = cquant
        self.price = float(0)


class Tmc(BaseArtixDomainEntity):
    """
    товар
    """

    def __init__(self, inventcode=None, barcode=None, name=None, price=None, measurecode=None, taxgroupcode=None,
                 articul=None, tmctype=None, barcodes=None, options=None, remain=None, discautoscheme=None,
                 inventgroup=None, minprice=None, taracapacity=None, deptcode=None, paymentobject=None, opmode=None):
        if barcodes is None:
            barcodes = []
        self.inventcode = inventcode
        self.barcode = barcode
        self.name = name
        self.price = price
        self.measurecode = measurecode
        self.taxgroupcode = taxgroupcode
        self.articul = articul
        self.tmctype = tmctype
        self.barcodes = barcodes
        self.options = options
        self.paymentobject = paymentobject
        self.remain = remain
        self.discautoscheme = discautoscheme
        self.inventgroup = inventgroup
        self.minprice = minprice
        self.taracapacity = taracapacity
        self.deptcode = deptcode
        self.opmode = opmode


class Price(BaseArtixDomainEntity):
    """
    переоценка
    """

    def __init__(self, barcode=None, price=None, minprice=None, minretailprice=None, effectivedate=None, documentid=None,
                 doctype=None):
        self.barcode = barcode
        self.price = price
        self.minprice = minprice
        self.minretailprice = minretailprice
        self.effectivedate = effectivedate if effectivedate else (
            datetime.combine(date.today(), datetime.min.time()).strftime("%Y-%m-%d %H:%M:%S"))
        self.documentid = documentid if documentid else 1
        self.doctype = doctype if doctype else 1


class Tprice(BaseArtixDomainEntity):
    """
    индексная цена
    """

    def __init__(self, pricecode=None, barcode=None, price=None, name=None, effectivedate=None, documentid=None):
        self.pricecode = pricecode
        self.barcode = barcode
        self.price = price
        self.name = name
        self.effectivedate = effectivedate if effectivedate else (
            datetime.combine(date.today(), datetime.min.time()).strftime("%Y-%m-%d %H:%M:%S"))
        self.documentid = documentid if documentid else 1


class AlcoCode(BaseArtixDomainEntity):
    """
    алкокод
    """

    def __init__(self, alcocode=None, tmccode=None):
        self.alcocode = alcocode
        self.tmccode = tmccode


class TmcAlcoCode(BaseArtixDomainEntity):
    """
    связка товар-алкокод
    """

    def __init__(self, tmccode=None, alcocode=None):
        self.tmccode = tmccode
        self.alcocode = alcocode


class TmcGroup(BaseArtixDomainEntity):
    """
    группа товаров
    """

    def __init__(self, groupcode=None, parentgroupcode=None, groupname=None):
        self.groupcode = groupcode
        self.parentgroupcode = parentgroupcode
        self.groupname = groupname


class TmcTax(BaseArtixDomainEntity):
    """
    налог
    """

    def __init__(self, code=None, name=None, rate=None, mode=None):
        self.code = code
        self.name = name
        self.rate = rate
        self.mode = mode


class TmcKitItem(BaseArtixDomainEntity):
    """
    товар из комплекта
    """

    def __init__(self, tmccode=None, tmckitbarcode=None, quantity=None):
        self.tmccode = tmccode
        self.tmckitbarcode = tmckitbarcode
        self.quantity = quantity


class TmcKit(BaseArtixDomainEntity):
    """
    комплекты товаров
    """

    def __init__(self, barcode=None, name=None):
        self.barcode = barcode
        self.name = name
